# -*- mode: python; coding: utf-8 -*-
#
# This example shows two labels, one at the center of the window with
# an animation on his opacity from 100 to 255, and another with the same
# size than the window, with an animation on his alignment to follow
# the window's borders.

from pgm import render, widget
from pgm.timing import controller, modifier, keyframe
from pgm.layout import window, group
from pgm.form import color, label
import sys, time, pgmrender, gobject

render.init('opengl1')

def key_press_cb(event, win):
    if event.keyval == pgmrender.KEY_ESCAPE:
        win.main_quit()
    if event.keyval == pgmrender.KEY_w:
        win.set_fullscreen(True)
    if event.keyval == pgmrender.KEY_x:
        win.set_fullscreen(False)

def resize_cb(event, win):
    win.regenerate_widgets()

def main(args):
    # Window creation
    win = window.Window(title='Animated labels',
                        color=color.Color((100,100,100,255)),
                        scale_mode=window.ADJUST, size=(800,600))

    # Labels creation
    lbl = label.Label(text='Caption', size=win.size,
                      weight=label.WEIGHT_BOLD, slant=label.SLANT_ITALIC,
                      font_size=30, color=(color.Color(color.WHITE)))
    lbl_centered = label.Label(text='Pigment', size=win.size, font_size=60,
                               color=(color.Color(color.YELLOW)),
                               weight=label.WEIGHT_BOLD,
                               slant=label.SLANT_ITALIC)

    # Add the widgets to the window
    win.add(lbl, lbl_centered)

    # Alignment animation
    align_timeline = [keyframe.KeyFrame((0.0, 0.0), 0.0),
                      keyframe.KeyFrame((1.0, 0.0), 0.25),
                      keyframe.KeyFrame((1.0, 1.0), 0.5),
                      keyframe.KeyFrame((0.0, 1.0), 0.75),
                      keyframe.KeyFrame((0.0, 0.0), 1.0)]
    align_modifier = modifier.Modifier([lbl], 'align', align_timeline)
    align_ctrl = controller.Controller(duration=15000, resolution=5,
                                       repeat_count=50,
                                       modifiers=[align_modifier])

    # Opacity animation
    opacity_timeline = [keyframe.KeyFrame(0, 0.0),
                        keyframe.KeyFrame(75, 1.0)]
    opacity_modifier = modifier.Modifier([lbl_centered], 'tz',
                                         opacity_timeline)
    opacity_ctrl = controller.Controller(duration=250,
                                         resolution=25,
                                         repeat_behavior=controller.REVERSE,
                                         repeat_count=controller.INFINITE,
                                         interpolation=controller.SINUSOIDAL,
                                         
                                         modifiers=[opacity_modifier])

    # Signal connections
    win.sig_key_press_event.connect(key_press_cb, win)
    win.sig_resize_event.connect(resize_cb, win)
    win.sig_exit_event.connect(lambda _: win.main_quit())

    # Start the controllers
    align_ctrl.start()
    opacity_ctrl.start()

    # Start the loop
    gobject.timeout_add(10, win.update)
    win.main()

# Entry point
if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))
