#include <pgmrender.h>

#define INCREMENT 0.5f

static PgmRenderBaseWindow *win = NULL;
static PgmRenderDrawable *drb = NULL;
static gboolean pressed = FALSE;
static gboolean dragged = FALSE;
static gint x_offset = 0;
static gint y_offset = 0;
static gfloat increment = INCREMENT;
static gboolean loop = FALSE;
static gboolean fullscreen = FALSE;

static gboolean is_over_surface (PgmRenderEventButton *button);
static void handle_event (GList *event);
static void rotate (void);

static gboolean
is_over_surface (PgmRenderEventButton *button)
{
  gint width, height, x, y;
  gboolean over;

  pgm_render_layout_get_size (PGM_RENDER_LAYOUT_IFACE (drb), &width, &height);
  pgm_render_layout_get_position (PGM_RENDER_LAYOUT_IFACE (drb), &x, &y);

  over = ((button->x > x && button->x < (x + width))
	  && (button->y > y && button->y < (y + height)));

  if (over)
    {
      x_offset = button->x - x;
      y_offset = button->y - y;
    }

  return over;
}

static void
handle_event (GList *event)
{
  while (event)
    {
      PgmRenderEvent *ev = event->data;

      switch (ev->type)
	{
	case PGM_RENDER_DELETE:
	  pgm_render_base_window_quit_loop (win);
	  loop = FALSE;
	  break;

	case PGM_RENDER_CONFIGURE:
	  break;

        case PGM_RENDER_KEY_PRESS:
          fullscreen = !fullscreen;
          if (fullscreen)
            pgm_render_base_window_set_fullscreen (win, fullscreen, 640, 480);
          else
            pgm_render_base_window_set_fullscreen (win, fullscreen, 800, 600);
          break;

	case PGM_RENDER_MOTION_NOTIFY:
	  if (pressed && dragged)
	    {
	      PgmRenderEventMotion *motion = (PgmRenderEventMotion*) ev;
	      pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb),
					      motion->x - x_offset,
					      motion->y - y_offset);
	      pgm_render_base_window_update (win);
	    }
	  break;

	case PGM_RENDER_BUTTON_PRESS:
	  if (is_over_surface ((PgmRenderEventButton*) ev))
	    {
	      g_print ("Start dragging. ");
	      dragged = TRUE;
	    }
	  pressed = TRUE;
	  break;

	case PGM_RENDER_BUTTON_RELEASE:
	  if (dragged)
	    {
	      g_print ("Stop dragging.\n");
	      dragged = FALSE;
	    }
	  pressed = FALSE;
	  break;

	default:
	  break;
	}

      g_free (ev);
      event = g_list_next (event);
    }

  g_list_free (event);
}

static void
rotate ()
{
  static gfloat rotate = 0.0f;

  rotate += increment;
  /* pgm_render_pipe3d_set_rotation (PGM_RENDER_PIPE3D_IFACE (drb), 0.0f, 0.0f, */
/*   				  rotate); */
  pgm_render_base_window_update (win);
}

int
main (int argc, char *argv[])
{
  const gchar *title = "Dragging";
  gint canvas_width, canvas_height;
  GList *event;

  /* Load the OpenGL 1.2 renderer plugin */
  pgm_render_init ("opengl1");

  /* Create the Window */
  win = pgm_render_create_window ();
  pgm_render_base_window_set_title (win, title, title);
  pgm_render_base_window_set_color (win, 200, 100, 0, 255);
  pgm_render_base_window_set_size (win, 800, 600);

  pgm_render_base_window_set_scale_mode (win, PGM_RENDER_WINDOW_SCREEN_ADJUST);
  pgm_render_base_window_get_canvas_size (win, &canvas_width, &canvas_height);
  g_print ("canvas %dx%d\n", canvas_width, canvas_height);

  /* Create and set up a Surface in the middle of the context */
  drb = pgm_render_create_surface ();
  pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb), 0, 100, 255, 175);
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb),
				  canvas_width/2.0f - 50,
				  canvas_height/2.0f - 50);

  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb), 100, 100);
/*   pgm_render_shape_set_arc (PGM_RENDER_SHAPE_IFACE (drb), 30.0f, 30.0f); */
/*   pgm_render_stroke_set_color (PGM_RENDER_STROKE_IFACE (drb), 255, 255, 255, */
/* 			       255); */
/*   pgm_render_stroke_set_width (PGM_RENDER_STROKE_IFACE (drb), 5.0f); */
/*   pgm_render_pipe3d_set_rotation (PGM_RENDER_PIPE3D_IFACE (drb), 0.0f, 0.0f, */
/* 				  90.0f); */
  /* Add the drawable and a rotating animation */
  pgm_render_base_window_add_drawable (win, drb, PGM_RENDER_DRAWABLE_Z_ORDERED);

  /* Start the main loop */
  pgm_render_base_window_start_loop (win);
  loop = TRUE;

  while (loop)
  {
    g_usleep (5000);
    pgm_render_base_window_get_events (win, &event);
    handle_event (event);
    rotate ();
  }

  /* Unload the plugin */
  pgm_render_deinit ();

  return 0;
}
