#include <pgmrender.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <unistd.h>
#include <stdlib.h>

#define INCREMENT 0.25f

static PgmRenderBaseWindow *win = NULL;
static PgmRenderDrawable *drb1, *drb2, *drb3, *drb4;
static PgmRenderLayoutAspectStyle style;
static gfloat increment = INCREMENT;
static gboolean loop = FALSE;
static gboolean fullscreen = FALSE;
gfloat w_cor, y_cor;

static void handle_event (GList *event);
static void rotate (void);

static void
handle_event (GList *event)
{
  static PgmRenderMediaInterpType type = PGM_RENDER_MEDIA_BILINEAR;

  while (event)
    {
      PgmRenderEvent *ev = event->data;

      switch (ev->type)
	{
	case PGM_RENDER_DELETE:
	  pgm_render_base_window_quit_loop (win);
	  loop = FALSE;
	  break;

/* 	case PGM_RENDER_CONFIGURE: */
/* 	  { */
/* 	    PgmRenderEventConfigure *conf = (PgmRenderEventConfigure*) ev; */
/* 	    pgm_render_base_window_set_canvas_size */
/* 	     (win, conf->width, conf->height); */
/* 	    pgm_render_base_window_set_scale (win, conf->width/800.0f, */
/* 					      conf->height/600.0f); */
/* 	    g_print ("%dx%d\n", conf->width, conf->height); */
/* 	  } */
/* 	  break; */

	case PGM_RENDER_BUTTON_PRESS:
	  pgm_render_layout_get_aspect_style
	    (PGM_RENDER_LAYOUT_IFACE (drb1), &style);
	  if (style == PGM_RENDER_LAYOUT_SCALED)
	    pgm_render_layout_set_aspect_style
	      (PGM_RENDER_LAYOUT_IFACE (drb1), PGM_RENDER_LAYOUT_FILLED);
	  else
	    pgm_render_layout_set_aspect_style
	      (PGM_RENDER_LAYOUT_IFACE (drb1), PGM_RENDER_LAYOUT_SCALED);
	  break;

	case PGM_RENDER_KEY_PRESS:
	  fullscreen = !fullscreen;
	  if (fullscreen)
	    pgm_render_base_window_set_fullscreen (win, TRUE, 1024, 768);
	  else
	    pgm_render_base_window_set_fullscreen (win, FALSE, 800, 600);
	  break;

	default:
	  break;
	}

      g_free (ev);
      event = g_list_next (event);
    }

  g_list_free (event);
}

static void
rotate ()
{
  static gfloat rotate = 0.0f;

  rotate += increment;
/*   pgm_render_pipe3d_set_rotation (PGM_RENDER_PIPE3D_IFACE (drb1), rotate*1.5f, */
/* 				  rotate, 0.0f); */
  pgm_render_base_window_update (win);
}

int
main (int argc, char *argv[])
{
  const gchar *title = "Image";
  GdkPixbuf *image = NULL;
  gpointer pixels = NULL;
  GList *event;
  PgmRenderMediaPixelFormat format;
  gint width, height, stride;

  if (argc != 2)
    {
      g_print ("Usage: %s image\n", argv[0]);
      exit (1);
    }

  /* Load the OpenGL 1.2 renderer plugin */
  pgm_render_init ("opengl1");

  /* Create the Window */
  win = pgm_render_create_window ();
  pgm_render_base_window_set_title (win, title, title);

  /* Load the image */
  image = gdk_pixbuf_new_from_file (argv[1], NULL);
  format = gdk_pixbuf_get_has_alpha (image) ?
    PGM_RENDER_MEDIA_RGBA : PGM_RENDER_MEDIA_RGB;
  width = gdk_pixbuf_get_width (image);
  height = gdk_pixbuf_get_height (image);
  stride = gdk_pixbuf_get_rowstride (image);
  pixels = (gpointer) gdk_pixbuf_get_pixels (image);
  g_print ("Colorspace:%s, Size:%dx%d, Stride:%d\n",
	   format == PGM_RENDER_MEDIA_RGBA ? "RGBA" : "RGB",
	   width, height, stride);

  /* Create and set up a Surface in the middle of the context */
  drb1 = pgm_render_create_surface ();
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb1), 200, 100);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb1), 400, 400);
  pgm_render_pipe3d_set_translation (PGM_RENDER_PIPE3D_IFACE (drb1),
				     0.0f, 0.0f, 0.0f);
  pgm_render_media_set_image (PGM_RENDER_MEDIA_IFACE (drb1), format, width,
			      height, stride, pixels);
  pgm_render_layout_set_aspect_style
    (PGM_RENDER_LAYOUT_IFACE (drb1), PGM_RENDER_LAYOUT_FILLED);
  style = PGM_RENDER_LAYOUT_FILLED;
  drb2 = pgm_render_create_surface ();
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb2), 125, 125);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb2), 200, 200);
  pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb2), 255, 0, 0, 105);
  pgm_render_pipe3d_set_translation (PGM_RENDER_PIPE3D_IFACE (drb2),
				     0.0f, 0.0f, 0.0f);

  drb3 = pgm_render_create_surface ();
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb3), 175, 175);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb3), 200, 200);
  pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb3), 0, 255, 0, 105);
  pgm_render_media_set_image_from_drawable (PGM_RENDER_MEDIA_IFACE (drb3), drb1);

  drb4 = pgm_render_create_surface ();
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb4), 225, 225);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb4), 200, 200);
  pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb4), 0, 0, 255, 105);

  /* Add the drawables */
  pgm_render_base_window_add_drawable (win, drb1, PGM_RENDER_DRAWABLE_FAR);
  pgm_render_base_window_add_drawable (win, drb2, PGM_RENDER_DRAWABLE_FAR);
  pgm_render_base_window_add_drawable (win, drb3, PGM_RENDER_DRAWABLE_FAR);
  pgm_render_base_window_add_drawable (win, drb4, PGM_RENDER_DRAWABLE_FAR);

  /* pgm_render_base_window_reorder_drawable (win, drb1, -1); */

  /* Start the main loop */
  pgm_render_base_window_start_loop (win);
  loop = TRUE;
  while (loop)
    {
      usleep (50000);
      pgm_render_base_window_get_events (win, &event);
      handle_event (event);
      rotate ();
    }

  /* Unload the plugin */
  pgm_render_deinit ();

  return 0;
}
